open Formula

let number_of_monomials_per_expression = ref 4;;
let min_int = ref (-20);;
let max_int = ref 20;;
let number_of_variables = ref 12
let variables_per_quantifier = ref 4
let formula_uq_depth = ref 6
let formula_uq_proba = ref 1.0
let and_proba = ref 0.9
let quant_depth = ref 12
let dense_formulas = ref false

let nth_variable n = "v" ^ (string_of_int n)
let random_variable () = nth_variable (Random.int !number_of_variables)

let gen_list f =
  let rec gen_list_rec already n =
    if n <= 0
    then already
    else gen_list_rec ((f ())::already) (n-1) in
    gen_list_rec [];;

let random_int () = Z.of_int
  ((Random.int (!max_int - !min_int + 1)) + !min_int)

let random_monomial () : source_expression = `Product
  [((`Integer (random_int ())) :> source_expression);
   ((`Variable (random_variable ())) :> source_expression)]

let random_linear_expression () : source_expression =
  if !dense_formulas
  then `Sum (Array.to_list (Array.init !number_of_variables
    (fun k ->
      `Product [ ((`Integer (random_int ())) :> source_expression);
	         ((`Variable (nth_variable k)) :> source_expression) ] )))
  else `Sum (gen_list random_monomial !number_of_monomials_per_expression);;
  
let random_atom () : source_quantified_formula =
  `Atomic (`Comparison((random_linear_expression ()),
		       `LESS_EQ,
		       (`Integer (random_int ()))));;

let rec random_uq_formula (depth : int) : source_quantified_formula =
  if depth <= 0 || Random.float 1.0 >= !formula_uq_proba
  then random_atom ()
  else let l = [ random_uq_formula (depth-1); random_uq_formula (depth-1)] in
    if Random.float 1.0 <= !and_proba
    then `And l
    else `Or l;;

let rec nth_variable_block cur_depth n =
  if n = 0 then []
  else (nth_variable (cur_depth-1))::(nth_variable_block (cur_depth-1) (n-1));;

let rec random_forall_formula (depth : int) : source_quantified_formula =
  if depth <= 0
  then random_uq_formula !formula_uq_depth
  else `Forall((nth_variable_block depth !variables_per_quantifier), random_exists_formula (depth - !variables_per_quantifier))

and random_exists_formula (depth : int) : source_quantified_formula =
  if depth <= 0
  then random_uq_formula !formula_uq_depth
  else `Exists((nth_variable_block depth !variables_per_quantifier), random_forall_formula (depth - !variables_per_quantifier));;

Random.init (int_of_string Sys.argv.(1));;

let formula = random_exists_formula (!quant_depth) in
  Formula_math.print_source_quantified_formula stdout formula;
  print_newline ()
